"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.App = void 0;
const assert_1 = __importDefault(require("assert"));
const chalk_1 = __importDefault(require("chalk"));
const root_scene_1 = require("./components/root-scene");
const package_json_1 = require("../package.json");
const summary_1 = require("./analystic/summary");
class App {
    logger;
    file;
    password;
    rootScene;
    summary;
    constructor(logger, file, password) {
        this.logger = logger;
        this.file = file;
        this.password = password;
        this.logger.log('%s\t%s', chalk_1.default.yellow(`${package_json_1.name}⚡️`), chalk_1.default.gray(`v${package_json_1.version}`));
        this.logger.log('');
        (0, assert_1.default)(this.file, 'Scene file is required');
        this.rootScene = new root_scene_1.RootScene({ path: this.file, password: this.password }, this.logger.clone('root-scene'));
        this.summary = new summary_1.Summary(this.rootScene.logger);
    }
    setDirTags(dirs) {
        this.logger.debug('External sources %j', dirs);
        this.rootScene.tagsManager.tagDirs = dirs;
    }
    async exec() {
        try {
            await this.rootScene?.exec();
        }
        finally {
            await this.rootScene?.dispose();
            this.summary.print();
        }
    }
}
exports.App = App;
//# sourceMappingURL=app.js.map