"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Job = void 0;
const assert_1 = __importDefault(require("assert"));
const file_store_1 = require("../../components/file/file-store");
const group_1 = require("../../components/group/group");
const jobs_manager_1 = require("../../libs/queue-jobs/jobs-manager");
const file_storage_1 = require("../../libs/storage/file-storage");
const time_1 = require("../../libs/time");
const job_executor_1 = require("./job-executor");
class Job extends group_1.Group {
    queue;
    jobsManager;
    constructor({ queue, ...props }) {
        super(props);
        Object.assign(this, { queue });
        this.$$ignoreEvalProps.push('jobsManager');
    }
    onJobInit(jobs) {
        return jobs?.map((job) => {
            return new job_executor_1.JobExecute(this.runEachOfElements.bind(this), job.data);
        }) || [];
    }
    async exec(input) {
        let t;
        if (this.queue) {
            const opts = {
                jobHandler: this,
                concurrent: this.queue?.concurrent || 1
            };
            if (this.queue.file) {
                opts.storage = new file_storage_1.FileStorage(this.logger, this.scene.getPath(this.queue.file), this.queue.password);
            }
            else if (this.queue.storage) {
                (0, assert_1.default)(this.queue.storage instanceof file_store_1.FileStore, 'Storage is not valid');
                opts.storage = this.queue.storage;
            }
            this.jobsManager = new jobs_manager_1.JobsManager(this.logger, opts);
            t = this.jobsManager.start();
            await (0, time_1.sleep)(500);
        }
        const proms = new Array();
        proms.push(this.execJob(input));
        if (t)
            proms.push(t);
        await Promise.all(proms);
        return [];
    }
    async addJobData(jobData) {
        if (!this.queue) {
            await this.runEachOfElements(jobData);
        }
        else {
            await this.jobsManager?.add(new job_executor_1.JobExecute(this.runEachOfElements.bind(this), jobData));
        }
    }
    async stop() {
        await this.jobsManager?.stop();
        this.jobsManager = undefined;
    }
    async dispose() {
        await this.stop();
    }
}
exports.Job = Job;
//# sourceMappingURL=job.js.map