"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.copyElementShadowPrototype = exports.ElementShadow = void 0;
const async_function_1 = require("./../libs/async-function");
const logger_1 = require("./../libs/logger");
const variable_1 = require("./../libs/variable");
const IGNORE_EVAL_ELEMENT_SHADOW_PROPS = [
    '$$evalExps',
    '$$ignoreVarsBeforeExec',
    'skip',
    'error',
    'vars',
    'loop',
    'loopKey',
    'loopValue',
    'if',
    '$$tag',
    'parent',
    'parentState',
    'result',
    'logger',
    'scene',
    'rootScene'
];
class ElementShadow {
    $$ignoreEvalProps = [];
    title;
    skip;
    force;
    error;
    log;
    vars;
    async;
    loop;
    loopKey;
    loopValue;
    if;
    $$tag;
    parentState = {};
    parent;
    result;
    logger;
    scene;
    rootScene;
    get $$loggerLevel() {
        return logger_1.Logger.LogLevel || this.log || this.parent?.$$loggerLevel || logger_1.LoggerLevel.ALL;
    }
    asyncConstructor(_props) { }
    getParentByClassName(...ClazzTypes) {
        let parent = this.parent;
        do {
            if (ClazzTypes.some(ClazzType => parent instanceof ClazzType)) {
                return parent;
            }
            parent = parent?.parent;
        } while (parent);
        return parent;
    }
    async setVarsAfterExec() {
        if (this.vars) {
            const keys = await this.scene.setVars(this.vars, this.result, this);
            if (!keys?.length)
                return;
            this.logger.trace('[vars] \t%j', keys.reduce((sum, e) => {
                sum[e] = this.scene.localVars[e];
                return sum;
            }, {}));
        }
    }
    async evalPropsBeforeExec() {
        const props = Object.keys(this);
        const proms = props
            .filter(key => {
            return !IGNORE_EVAL_ELEMENT_SHADOW_PROPS.includes(key) &&
                !this.$$ignoreEvalProps?.includes(key) &&
                (0, variable_1.isGetEvalExp)(this[key]);
        }).map(async (key) => {
            this[key] = await this.scene.getVars(this[key], this);
        });
        proms.length && await Promise.all(proms);
    }
    async callFunctionScript(script, ctx = this, others = {}) {
        const rs = await (0, async_function_1.callFunctionScript)(script, ctx, {
            logger: this.logger,
            vars: this.scene.localVars,
            utils: this.rootScene.globalUtils,
            ...others
        });
        return rs;
    }
    dispose() { }
    disposeApp;
}
exports.ElementShadow = ElementShadow;
const ownerProperties = Object.getOwnPropertyNames(ElementShadow.prototype).filter(k => !['constructor', 'exec', 'dispose'].includes(k));
function copyElementShadowPrototype(elem) {
    if (elem.$$loggerLevel)
        return;
    if (!elem.$$ignoreEvalProps)
        elem.$$ignoreEvalProps = [];
    ownerProperties.forEach(k => {
        if (!elem[k])
            elem[k] = ElementShadow.prototype[k];
    });
}
exports.copyElementShadowPrototype = copyElementShadowPrototype;
//# sourceMappingURL=element-shadow.js.map