"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.ExecSh = void 0;
const assert_1 = __importDefault(require("assert"));
const chalk_1 = __importDefault(require("chalk"));
const child_process_1 = require("child_process");
const file_remote_1 = require("../../libs/file-remote");
const file_temp_1 = require("../../libs/file-temp");
const element_shadow_1 = require("../element-shadow");
class ExecSh extends element_shadow_1.ElementShadow {
    script;
    path;
    bin = '/bin/sh';
    constructor(props) {
        super();
        if (typeof props === 'string') {
            props = {
                script: props
            };
        }
        Object.assign(this, props);
    }
    async exec() {
        if (this.path) {
            const fileRemote = new file_remote_1.FileRemote(this.path, this.scene);
            this.script = await fileRemote.getTextContent();
        }
        (0, assert_1.default)(this.script);
        const tmpFile = new file_temp_1.FileTemp();
        try {
            await tmpFile.create(this.script);
            const rs = await new Promise((resolve, reject) => {
                (0, child_process_1.execFile)(this.bin, [tmpFile.file], { env: process.env, cwd: this.scene.curDir }, (err, stdout, stderr) => {
                    if (err)
                        return reject(err);
                    stdout && this.logger.debug(chalk_1.default.gray(stdout));
                    stderr && this.logger.debug(chalk_1.default.red(stderr));
                    resolve((stdout + '\r\n' + stderr).trim());
                });
            });
            return rs;
        }
        finally {
            tmpFile.remove();
        }
    }
}
exports.ExecSh = ExecSh;
//# sourceMappingURL=exec-sh.js.map