"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Group = void 0;
const lodash_clonedeep_1 = __importDefault(require("lodash.clonedeep"));
const continue_1 = require("../continue/continue");
const element_shadow_1 = require("../element-shadow");
class Group extends element_shadow_1.ElementShadow {
    $$ignoreEvalProps = ['runs'];
    runs = [];
    get innerScene() {
        return this.scene;
    }
    constructor(props) {
        super();
        this.lazyInit(props);
    }
    lazyInit(props) {
        if (Array.isArray(props)) {
            props = {
                runs: props
            };
        }
        Object.assign(this, props);
        this.runs = this.runs?.filter(e => e) || [];
    }
    async newElement(nameOrClass, props) {
        let elem;
        if (typeof nameOrClass === 'string') {
            const name = nameOrClass;
            const ElemClass = await this.rootScene.tagsManager.loadElementClass(name, this.innerScene);
            elem = this.innerScene.elementBuilder
                .element(ElemClass, props)
                .tag(name)
                .parent(this)
                .logger(this.logger)
                .build();
        }
        else {
            const ElemClass = nameOrClass;
            elem = await this.innerScene.elementBuilder
                .element(ElemClass, props)
                .parent(this)
                .logger(this.logger)
                .build();
        }
        return elem;
    }
    async exec(parentState) {
        return await this.runEachOfElements(parentState);
    }
    async runEachOfElements(parentState) {
        const asyncJobs = new Array();
        const runs = new Array();
        const newRuns = (0, lodash_clonedeep_1.default)(this.runs);
        for (const _e of newRuns) {
            const e = typeof _e === 'string' ? { echo: { content: _e } } : _e;
            const name = Object.keys(e)[0];
            let props = e[name];
            let baseProps = {};
            if (props && typeof props === 'object' && !Array.isArray(props)) {
                const { '<-': inheritKeys, '->': exposeKey, skip, ...eProps } = props;
                if (inheritKeys)
                    this.rootScene.extend(eProps, inheritKeys);
                if (exposeKey)
                    this.rootScene.export(exposeKey, eProps);
                const { if: condition, force, log, vars, async, loop, ...customProps } = eProps;
                baseProps = {
                    if: condition,
                    force,
                    log,
                    vars,
                    async,
                    loop
                };
                props = customProps;
                if (skip)
                    continue;
            }
            const loop = baseProps.loop;
            if (loop === undefined) {
                const elem = await this.createAndExecuteElement(asyncJobs, name, parentState, baseProps, props);
                if (elem) {
                    runs.push(elem);
                    if (elem instanceof continue_1.Continue)
                        break;
                }
            }
            else {
                let loopCondition = await this.innerScene.getVars(loop, this);
                if (loopCondition) {
                    if (Array.isArray(loopCondition)) {
                        for (let i = 0; i < loopCondition.length; i++) {
                            const newProps = props && (0, lodash_clonedeep_1.default)(props);
                            const newElem = await this.createAndExecuteElement(asyncJobs, name, parentState, baseProps, newProps, {
                                loopKey: i,
                                loopValue: loopCondition[i]
                            });
                            if (newElem) {
                                runs.push(newElem);
                                if (newElem instanceof continue_1.Continue)
                                    break;
                            }
                        }
                    }
                    else if (typeof loopCondition === 'object') {
                        for (const key in loopCondition) {
                            const newProps = props && (0, lodash_clonedeep_1.default)(props);
                            const newElem = await this.createAndExecuteElement(asyncJobs, name, parentState, baseProps, newProps, {
                                loopKey: key,
                                loopValue: loopCondition[key]
                            });
                            if (newElem) {
                                runs.push(newElem);
                                if (newElem instanceof continue_1.Continue)
                                    break;
                            }
                        }
                    }
                    else if (loopCondition === true) {
                        while (loopCondition) {
                            const newProps = props && (0, lodash_clonedeep_1.default)(props);
                            const newElem = await this.createAndExecuteElement(asyncJobs, name, parentState, baseProps, newProps, {
                                loopValue: loopCondition
                            });
                            if (newElem) {
                                runs.push(newElem);
                                if (newElem instanceof continue_1.Continue)
                                    break;
                            }
                            loopCondition = await this.innerScene.getVars(loop, this);
                        }
                    }
                }
            }
        }
        if (asyncJobs.length) {
            await Promise.all(asyncJobs);
        }
        return runs;
    }
    async createAndExecuteElement(asyncJobs, name, parentState, baseProps, props, loopObj = {}) {
        const isContinue = (baseProps.if === undefined) || await this.innerScene.getVars(baseProps.if, { parent: this, parentState, ...loopObj }, { parentState });
        if (!isContinue)
            return undefined;
        if (asyncJobs.length && !baseProps?.async) {
            await Promise.all(asyncJobs);
            asyncJobs.splice(0, asyncJobs.length);
        }
        const elem = await this.newElement(name, props);
        Object.assign(elem, loopObj, baseProps);
        const p = elem.exec(parentState).finally(() => elem.dispose());
        if (!elem.async) {
            await p;
        }
        else {
            asyncJobs.push(p);
        }
        return elem;
    }
}
exports.Group = Group;
//# sourceMappingURL=group.js.map