"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Pub = void 0;
const assert_1 = __importDefault(require("assert"));
const chalk_1 = __importDefault(require("chalk"));
const element_shadow_1 = require("../../../components/element-shadow");
class Pub extends element_shadow_1.ElementShadow {
    address;
    data;
    secure;
    constructor(props) {
        super();
        Object.assign(this, props);
    }
    async exec() {
        try {
            (0, assert_1.default)(this.address);
            const address = this.address.startsWith('http://') ? this.address : `http://${this.address}`;
            this.logger.debug('Add a job').trace('%j', this.data);
            const headers = { 'content-type': 'application/json' };
            if (this.secure) {
                if (typeof this.secure === 'string') {
                    headers.authorization = this.secure;
                }
                else if (this.secure.basic) {
                    headers.authorization = `Basic ${this.secure.basic}`;
                }
            }
            const rs = await fetch(address.toString(), {
                method: 'post',
                headers,
                body: this.data ? JSON.stringify(this.data) : undefined
            });
            return rs;
        }
        catch (err) {
            this.logger.warn(`Could not add a job to queue "${this.address}" \t %s`, chalk_1.default.gray(err?.message));
        }
    }
}
exports.Pub = Pub;
//# sourceMappingURL=pub.js.map