"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.RootScene = void 0;
const lodash_clonedeep_1 = __importDefault(require("lodash.clonedeep"));
const lodash_merge_1 = __importDefault(require("lodash.merge"));
const scene_1 = require("./../components/scene/scene");
const events_manager_1 = require("./../managers/events-manager");
const tags_manager_1 = require("./../managers/tags-manager");
const templates_manager_1 = require("./../managers/templates-manager");
const utility_function_manager_1 = require("./../managers/utility-function-manager");
class RootScene extends scene_1.Scene {
    logger;
    tagsManager = new tags_manager_1.TagsManager(this);
    templatesManager = new templates_manager_1.TemplatesManager();
    globalUtils = new utility_function_manager_1.UtilityFunctionManager();
    globalVars = {};
    disposeApps = new Array();
    runDir = process.cwd();
    rootDir = '';
    constructor(props, logger) {
        super(props);
        this.logger = logger;
        this.$$ignoreEvalProps.push('globalUtils', 'tagsManager', 'globalVars', 'templatesManager', 'rootDir');
        this.log = this.logger.levelName;
        this.isRoot = true;
        this.scene = this.rootScene = this;
    }
    async exec() {
        await this.asyncConstructor();
        events_manager_1.GlobalEvent.emit('scene/exec:before');
        try {
            const rs = await super.exec();
            return rs;
        }
        finally {
            events_manager_1.GlobalEvent.emit('scene/exec:end');
        }
    }
    async dispose() {
        events_manager_1.GlobalEvent.emit('scene/dispose:before');
        try {
            const proms = this.disposeApps.map(async (elem) => {
                if (elem.disposeApp !== undefined) {
                    await elem.disposeApp();
                }
            });
            if (proms.length) {
                await Promise.all(proms);
            }
            await super.dispose();
        }
        finally {
            events_manager_1.GlobalEvent.emit('scene/dispose:end');
        }
    }
    extend(baseProps, ids) {
        if (!ids?.length)
            return;
        if (typeof ids === 'string')
            ids = [ids];
        ids.forEach(id => {
            let cached = this.templatesManager.getFromCached(id);
            if (!cached) {
                this.logger.warn(`Could not found element with id "${id}"`);
                cached = {
                    skip: true
                };
            }
            (0, lodash_merge_1.default)(baseProps, (0, lodash_clonedeep_1.default)(cached));
        });
    }
    export(id, props) {
        if (!id)
            return;
        this.templatesManager.pushToCached(id, (0, lodash_clonedeep_1.default)(props));
        this.logger.trace('->    \t%s', id);
    }
}
exports.RootScene = RootScene;
//# sourceMappingURL=root-scene.js.map