"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const assert_1 = __importDefault(require("assert"));
const chalk_1 = __importDefault(require("chalk"));
const commander_1 = require("commander");
require("./managers/modules-manager");
const package_json_1 = require("../package.json");
const app_1 = require("./app");
const root_scene_1 = require("./components/root-scene");
const logger_1 = require("./libs/logger");
const packages_manager_1 = require("./managers/packages-manager");
commander_1.program
    .name(package_json_1.name)
    .aliases(Object.keys(package_json_1.bin).filter(e => e !== package_json_1.name))
    .description(package_json_1.description)
    .version(package_json_1.version, '-v, --version')
    .argument('<file>', 'scenario path or file')
    .argument('[password]', 'password to decrypt scene file')
    .enablePositionalOptions(true)
    .passThroughOptions(true)
    .showHelpAfterError(true)
    .option('--log <level>', 'set log details ("all", "trace", "debug", "info", "warn", "error", "fatal", "silent")')
    .option('--tagDirs <path...>', 'path to folder which includes external tags')
    .option('-e, --env <key=value...>', 'environment variables')
    .action(async (source, password, opts = {}) => {
    try {
        await fetch('');
    }
    catch { }
    const { log, env, tagDirs } = opts;
    env?.filter((keyValue) => keyValue.includes('='))
        .forEach((keyValue) => {
        const idx = keyValue.indexOf('=');
        const key = keyValue.substring(0, idx);
        const vl = keyValue.substring(idx + 1);
        process.env[key] = vl;
    });
    if (log)
        logger_1.Logger.LogLevel = log;
    const appLogger = new logger_1.Logger(log ?? logger_1.LoggerLevel.INFO);
    const app = new app_1.App(appLogger, source, password);
    tagDirs?.length && app.setDirTags(tagDirs);
    await app.exec();
})
    .addHelpText('after', () => {
    const { dependencies = {} } = require('./package.json');
    const msg = ['Sub modules version'];
    Object.keys(dependencies).forEach(key => msg.push(`- ${chalk_1.default.green(key)}${chalk_1.default.gray(dependencies[key])}`));
    return msg.length > 1 ? msg.join('\n') : '';
})
    .addCommand(commander_1.program
    .createCommand('add')
    .aliases(['install'])
    .description('add external tags version')
    .argument('[package_name...]', 'packages in npm registry')
    .action(async (packages) => {
    const appLogger = new logger_1.Logger(logger_1.LoggerLevel.ALL);
    await new packages_manager_1.PackagesManager(new root_scene_1.RootScene({}, appLogger)).install(...packages);
    process.exit(0);
}))
    .addCommand(commander_1.program
    .createCommand('up')
    .aliases(['upgrade', 'update'])
    .description('upgrade external tags version')
    .argument('[package_name...]', 'packages in npm registry')
    .action(async (packages) => {
    const appLogger = new logger_1.Logger(logger_1.LoggerLevel.ALL);
    await new packages_manager_1.PackagesManager(new root_scene_1.RootScene({}, appLogger)).upgrade(...packages);
    process.exit(0);
}))
    .addCommand(commander_1.program
    .createCommand('rm')
    .aliases(['remove', 'delete'])
    .description('remove external tags version')
    .argument('[package_name...]', 'packages in npm registry')
    .action(async (packages) => {
    (0, assert_1.default)(packages?.length, '"package(s)" is requried');
    const appLogger = new logger_1.Logger(logger_1.LoggerLevel.ALL);
    await new packages_manager_1.PackagesManager(new root_scene_1.RootScene({}, appLogger)).uninstall(...packages);
    process.exit(0);
}))
    .addHelpText('after', `More:
✔ Github project: ${package_json_1.homepage}
✔ Npm package   : https://www.npmjs.com/package/${package_json_1.name}
✔ Docker Image  : https://hub.docker.com/repository/docker/doanthuanthanh88/${package_json_1.name}
`)
    .parse(process.argv);
//# sourceMappingURL=index.js.map