"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.FileRemote = void 0;
const assert_1 = __importDefault(require("assert"));
const fs_1 = require("fs");
const promises_1 = require("fs/promises");
class FileRemote {
    uri;
    get isRemote() {
        return !!((this.uri.startsWith('http://') || this.uri.startsWith('https://')));
    }
    constructor(uri, scene) {
        this.uri = uri;
        if (!this.isRemote) {
            this.uri = scene.getPath(this.uri);
        }
        (0, assert_1.default)(this.uri, 'File is required');
    }
    get existed() {
        if (this.isRemote)
            return undefined;
        return (0, fs_1.existsSync)(this.uri);
    }
    async getContent() {
        if (!this.isRemote) {
            const buf = await (0, promises_1.readFile)(this.uri);
            return buf;
        }
        const rs = await fetch(this.uri);
        const ab = await rs.arrayBuffer();
        return Buffer.from(new Uint8Array(ab));
    }
    async getTextContent() {
        if (!this.isRemote) {
            const buf = await (0, promises_1.readFile)(this.uri);
            return buf.toString();
        }
        const rs = await fetch(this.uri);
        const txt = await rs.text();
        return txt;
    }
}
exports.FileRemote = FileRemote;
//# sourceMappingURL=file-remote.js.map