"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.ProgressBar = void 0;
const spinnies_1 = __importDefault(require("spinnies"));
const time_1 = require("./time");
const spinnies = new spinnies_1.default();
class ProgressBar {
    logger;
    color;
    static Color = {
        Black: 'black',
        Red: 'red',
        Green: 'green',
        Yellow: 'yellow',
        Blue: 'blue',
        Magenta: 'magenta',
        Cyan: 'cyan',
        White: 'white',
        Gray: 'gray'
    };
    id = `${Date.now()}-${Math.random()}`;
    maxLength = 0;
    constructor(logger, color = ProgressBar.Color.Cyan) {
        this.logger = logger;
        this.color = color;
    }
    async start(txt) {
        if (txt.length > this.maxLength)
            this.maxLength = txt.length;
        spinnies.add(this.id, {
            text: txt,
            indent: this.logger.indent * 2,
            color: this.color
        });
    }
    update(txt, level) {
        if (txt.length > this.maxLength)
            this.maxLength = txt.length;
        spinnies.update(this.id, {
            text: this.logger.formatWithoutIndent(txt, level)
        });
    }
    async stop() {
        this.update(new Array(this.maxLength).fill(' ').join(''));
        await (0, time_1.sleep)(100);
        spinnies.remove(this.id);
    }
    async passed(txt, ...args) {
        await this.stop();
        this.logger.info(txt, ...args);
    }
    async failed(txt, ...args) {
        await this.stop();
        this.logger.error(txt, ...args);
    }
}
exports.ProgressBar = ProgressBar;
//# sourceMappingURL=progress-bar.js.map