"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.TagsManager = exports.ClassInFileCharacter = void 0;
const assert_1 = __importDefault(require("assert"));
const path_1 = require("path");
const time_1 = require("./../libs/time");
const packages_manager_1 = require("./packages-manager");
exports.ClassInFileCharacter = "'";
class TagsManager {
    scene;
    modules = {};
    tags = {};
    tagDirs = [];
    lockInstall = false;
    packages = new Set();
    constructor(scene) {
        this.scene = scene;
    }
    register(name, pathOfModule) {
        this.modules[name] = this.scene.getPath(pathOfModule);
    }
    setTag(name, obj) {
        this.tags[name] = obj;
    }
    async getTag(name) {
        var _a;
        const path = this.modules[name];
        let isExit;
        try {
            (0, assert_1.default)(path, `Could not found tag module "${name}"`);
            isExit = true;
            const Clazz = await (_a = path, Promise.resolve().then(() => __importStar(require(_a))));
            return Clazz;
        }
        catch (err) {
            const tag = this.tags[name];
            if (!tag) {
                err.$$exit = isExit;
                throw err;
            }
            return tag;
        }
    }
    reset() {
        this.modules = {};
    }
    async install(...packages) {
        while (this.lockInstall) {
            await (0, time_1.sleep)(1000);
        }
        this.lockInstall = true;
        try {
            packages.forEach(pack => this.packages.add(pack));
            this.scene.logger.debug('Preparing to install the lack packages...');
            const packagesManager = new packages_manager_1.PackagesManager(this.scene);
            await packagesManager.install(...Array.from(this.packages));
            this.packages.clear();
        }
        finally {
            this.lockInstall = false;
        }
    }
    async loadElementClass(name, scene) {
        var _a, _b, _c;
        let ElementModule;
        const [path, className = 'default'] = name.split(exports.ClassInFileCharacter);
        let triedToInstall;
        do {
            try {
                try {
                    ElementModule = await (_a = `../components/${path}`, Promise.resolve().then(() => __importStar(require(_a))));
                }
                catch (err) {
                    for (const dir of this.tagDirs) {
                        try {
                            ElementModule = await (_b = this.scene.getPath((0, path_1.join)(dir, path)), Promise.resolve().then(() => __importStar(require(_b))));
                        }
                        catch { }
                    }
                    if (!ElementModule)
                        throw err;
                }
            }
            catch (err1) {
                try {
                    ElementModule = await this.getTag(path);
                }
                catch (err2) {
                    if (err2.$$exit)
                        throw err2;
                    try {
                        ElementModule = await (_c = path, Promise.resolve().then(() => __importStar(require(_c))));
                    }
                    catch (err3) {
                        if (triedToInstall) {
                            scene.logger.error(1, err1?.message);
                            scene.logger.error(2, err2?.message);
                            scene.logger.error(3, err3?.message);
                            throw new Error(`Could not found class "${className}" in "${path}"`);
                        }
                        triedToInstall = true;
                        scene.logger.warn(1, err1?.message);
                        scene.logger.warn(2, err2?.message);
                        scene.logger.warn(3, err3?.message);
                        await this.install(path);
                    }
                }
            }
        } while (!ElementModule);
        let ElementClazz = ElementModule[className];
        if (ElementClazz && !ElementClazz.prototype) {
            if (typeof ElementClazz === 'function') {
                try {
                    ElementClazz = await ElementClazz();
                }
                catch (err) {
                    err.$$exit = true;
                    throw err;
                }
            }
            else {
                const { constructor, ...objProps } = ElementClazz;
                ElementClazz = class UnknownTag {
                    $$tag = name;
                    constructor(props) {
                        constructor?.call(this, props);
                    }
                };
                Object.assign(ElementClazz.prototype, objProps);
            }
        }
        (0, assert_1.default)(ElementClazz?.prototype, `Could not found the tag "${path}.${className}"`);
        return ElementClazz;
    }
}
exports.TagsManager = TagsManager;
//# sourceMappingURL=tags-manager.js.map